struct ComputeTask_BaseState <: AbstractComputeTask end         # calculate the base state of an external particle
struct ComputeTask_Propagator <: AbstractComputeTask end        # calculate the propagator term of a virtual particle
struct ComputeTask_Pair <: AbstractComputeTask end              # from a pair of virtual particle currents, calculate the product
struct ComputeTask_CollectPairs <: AbstractComputeTask end      # for a list of virtual particle current pair products and a propagator, sum and propagate
struct ComputeTask_Triple <: AbstractComputeTask end            # from a triple of virtual particle currents, calculate the diagram result
struct ComputeTask_CollectTriples <: AbstractComputeTask end    # sum over triples results and 

struct BaseStateInput{PS_T<:AbstractParticleStateful,SPIN_POL_T<:AbstractSpinOrPolarization}
    particle::PS_T
    spin_pol::SPIN_POL_T
end

function MetagraphOptimization.compute(::ComputeTask_BaseState, input::BaseStateInput{PS,SPIN_POL}) where {PS,SPIN_POL}
    return QEDbase.base_state(particle_species(input.particle), particle_direction(input.particle), momentum(input.particle), input.spin_pol)
end

struct PropagatorInput{VP_T<:VirtualParticle,PSP_T<:AbstractPhaseSpacePoint}
    vp::VP_T
    psp::PSP_T
end

function MetagraphOptimization.compute(::ComputeTask_Propagator, input::PropagatorInput{VP_T,PSP_T}) where {VP_T,PSP_T}
    vp_mom = zero(typeof(momentum(input.psp, Incoming(), 1)))
    for i in eachindex(in_contributions(input.vp))
        if in_contributions(input.vp)[i]
            vp_mom += momentum(input.psp, Incoming(), i)
        end
    end
    for o in eachindex(out_contributions(input.vp))
        if (out_contributions(input.vp))[o]
            vp_mom -= momentum(input.psp, Outgoing(), o)
        end
    end

    vp_species = particle_species(input.vp)
    vp_mass = mass(vp_species)
    return QEDbase.propagator(vp_species, vp_mom)
end
